@@ -13,6 +13,9 @@
 
 """Client for V1 API."""
 
+import logging
+import time
+
 import six
 
 from ironic_inspector_client.common import http
@@ -20,7 +23,18 @@ from ironic_inspector_client.common.i18n import _
 
 
 DEFAULT_API_VERSION = (1, 0)
-MAX_API_VERSION = (1, 0)
+MAX_API_VERSION = (1, 4)
+
+# using huge timeout by default, as precise timeout should be set in
+# ironic-inspector settings
+DEFAULT_RETRY_INTERVAL = 10
+DEFAULT_MAX_RETRIES = 3600
+
+LOG = logging.getLogger(__name__)
+
+
+class WaitTimeoutError(Exception):
+    """Timeout while waiting for nodes to finish introspection."""
 
 
 class ClientV1(http.BaseClient):
@@ -60,6 +74,23 @@ class ClientV1(http.BaseClient):
                   'new_ipmi_password': new_ipmi_password}
         self.request('post', '/introspection/%s' % uuid, params=params)
 
+    def reprocess(self, uuid):
+        """Reprocess stored introspection data.
+
+        :param uuid: node UUID.
+        :raises: ClientError on error reported from the server.
+        :raises: VersionNotSupported if requested api_version is not supported.
+        :raises: *requests* library exception on connection problems.
+        :raises: TypeError if uuid is not a string.
+        """
+        if not isinstance(uuid, six.string_types):
+            raise TypeError(_("Expected string for uuid argument, got"
+                              " %r instead") % uuid)
+
+        return self.request('post',
+                            '/introspection/%s/data/unprocessed' %
+                            uuid)
+
     def get_status(self, uuid):
         """Get introspection status for a node.
 
@@ -67,6 +98,8 @@ class ClientV1(http.BaseClient):
         :raises: ClientError on error reported from a server
         :raises: VersionNotSupported if requested api_version is not supported
         :raises: *requests* library exception on connection problems.
+        :return: dictionary with keys "finished" (True/False) and "error"
+                 (error string or None).
         """
         if not isinstance(uuid, six.string_types):
             raise TypeError(
@@ -74,6 +107,49 @@ class ClientV1(http.BaseClient):
 
         return self.request('get', '/introspection/%s' % uuid).json()
 
+    def wait_for_finish(self, uuids, retry_interval=DEFAULT_RETRY_INTERVAL,
+                        max_retries=DEFAULT_MAX_RETRIES,
+                        sleep_function=time.sleep):
+        """Wait for introspection finishing for given nodes.
+
+        :param uuids: collection of node uuid's.
+        :param retry_interval: sleep interval between retries.
+        :param max_retries: maximum number of retries.
+        :param sleep_function: function used for sleeping between retries.
+        :raises: WaitTimeoutError on timeout
+        :raises: ClientError on error reported from a server
+        :raises: VersionNotSupported if requested api_version is not supported
+        :raises: *requests* library exception on connection problems.
+        :return: dictionary UUID -> status (the same as in get_status).
+        """
+        result = {}
+
+        # Number of attempts = number of retries + first attempt
+        for attempt in range(max_retries + 1):
+            new_active_uuids = []
+            for uuid in uuids:
+                status = self.get_status(uuid)
+                if status.get('finished'):
+                    result[uuid] = status
+                else:
+                    new_active_uuids.append(uuid)
+
+            if new_active_uuids:
+                if attempt != max_retries:
+                    uuids = new_active_uuids
+                    LOG.debug('Still waiting for introspection results for '
+                              '%(count)d nodes, attempt %(attempt)d of '
+                              '%(total)d',
+                              {'count': len(new_active_uuids),
+                               'attempt': attempt + 1,
+                               'total': max_retries + 1})
+                    sleep_function(retry_interval)
+            else:
+                return result
+
+        raise WaitTimeoutError(_("Timeout while waiting for introspection "
+                                 "of nodes %s") % new_active_uuids)
+
     def get_data(self, uuid, raw=False):
         """Get introspection data from the last introspection of a node.
 
@@ -95,6 +171,21 @@ class ClientV1(http.BaseClient):
         else:
             return resp.json()
 
+    def abort(self, uuid):
+        """Abort running introspection for a node.
+
+        :param uuid: node UUID.
+        :raises: ClientError on error reported from a server.
+        :raises: VersionNotSupported if requested api_version is not supported.
+        :raises: *requests* library exception on connection problems.
+        :raises: TypeError if uuid is not a string.
+        """
+        if not isinstance(uuid, six.string_types):
+            raise TypeError(_("Expected string for uuid argument, got"
+                              " %r") % uuid)
+
+        return self.request('post', '/introspection/%s/abort' % uuid)
+
 
 class _RulesAPI(object):
     """Introspection rules API."""
